@@ -33,7 +33,7 @@ import (
 	routev1 "github.com/openshift/api/route/v1"
 	securityv1 "github.com/openshift/api/security/v1"
 	"github.com/pkg/errors"
-	appsv1 "k8s.io/api/apps/v1beta2"
+	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/api/extensions/v1beta1"
 	rbacv1 "k8s.io/api/rbac/v1"
@@ -95,8 +95,22 @@ var (
 	PrometheusK8sEtcdServiceMonitor       = "assets/prometheus-k8s/service-monitor-etcd.yaml"
 	PrometheusK8sServingCertsCABundle     = "assets/prometheus-k8s/serving-certs-ca-bundle.yaml"
 	PrometheusK8sKubeletServingCABundle   = "assets/prometheus-k8s/kubelet-serving-ca-bundle.yaml"
+	PrometheusK8sGrpcTLSSecret            = "assets/prometheus-k8s/grpc-tls-secret.yaml"
 	PrometheusK8sTrustedCABundle          = "assets/prometheus-k8s/trusted-ca-bundle.yaml"
 
+	PrometheusUserWorkloadServingCertsCABundle     = "assets/prometheus-user-workload/serving-certs-ca-bundle.yaml"
+	PrometheusUserWorkloadServiceAccount           = "assets/prometheus-user-workload/service-account.yaml"
+	PrometheusUserWorkloadClusterRole              = "assets/prometheus-user-workload/cluster-role.yaml"
+	PrometheusUserWorkloadClusterRoleBinding       = "assets/prometheus-user-workload/cluster-role-binding.yaml"
+	PrometheusUserWorkloadRoleConfig               = "assets/prometheus-user-workload/role-config.yaml"
+	PrometheusUserWorkloadRoleList                 = "assets/prometheus-user-workload/role-specific-namespaces.yaml"
+	PrometheusUserWorkloadRoleBindingList          = "assets/prometheus-user-workload/role-binding-specific-namespaces.yaml"
+	PrometheusUserWorkloadRoleBindingConfig        = "assets/prometheus-user-workload/role-binding-config.yaml"
+	PrometheusUserWorkloadService                  = "assets/prometheus-user-workload/service.yaml"
+	PrometheusUserWorkload                         = "assets/prometheus-user-workload/prometheus.yaml"
+	PrometheusUserWorkloadPrometheusServiceMonitor = "assets/prometheus-user-workload/service-monitor.yaml"
+	PrometheusUserWorkloadGrpcTLSSecret            = "assets/prometheus-user-workload/grpc-tls-secret.yaml"
+
 	PrometheusAdapterAPIService                         = "assets/prometheus-adapter/api-service.yaml"
 	PrometheusAdapterClusterRole                        = "assets/prometheus-adapter/cluster-role.yaml"
 	PrometheusAdapterClusterRoleBinding                 = "assets/prometheus-adapter/cluster-role-binding.yaml"
@@ -118,6 +132,13 @@ var (
 	PrometheusOperatorService            = "assets/prometheus-operator/service.yaml"
 	PrometheusOperatorServiceMonitor     = "assets/prometheus-operator/service-monitor.yaml"
 
+	PrometheusOperatorUserWorkloadServiceAccount     = "assets/prometheus-operator-user-workload/service-account.yaml"
+	PrometheusOperatorUserWorkloadClusterRole        = "assets/prometheus-operator-user-workload/cluster-role.yaml"
+	PrometheusOperatorUserWorkloadClusterRoleBinding = "assets/prometheus-operator-user-workload/cluster-role-binding.yaml"
+	PrometheusOperatorUserWorkloadService            = "assets/prometheus-operator-user-workload/service.yaml"
+	PrometheusOperatorUserWorkloadDeployment         = "assets/prometheus-operator-user-workload/deployment.yaml"
+	PrometheusOperatorUserWorkloadServiceMonitor     = "assets/prometheus-operator-user-workload/service-monitor.yaml"
+
 	GrafanaClusterRoleBinding   = "assets/grafana/cluster-role-binding.yaml"
 	GrafanaClusterRole          = "assets/grafana/cluster-role.yaml"
 	GrafanaConfigSecret         = "assets/grafana/config.yaml"
@@ -135,6 +156,7 @@ var (
 	ClusterMonitoringOperatorService        = "assets/cluster-monitoring-operator/service.yaml"
 	ClusterMonitoringOperatorServiceMonitor = "assets/cluster-monitoring-operator/service-monitor.yaml"
 	ClusterMonitoringClusterRole            = "assets/cluster-monitoring-operator/cluster-role.yaml"
+	ClusterMonitoringGrpcTLSSecret          = "assets/cluster-monitoring-operator/grpc-tls-secret.yaml"
 
 	TelemeterClientClusterRole            = "assets/telemeter-client/cluster-role.yaml"
 	TelemeterClientClusterRoleBinding     = "assets/telemeter-client/cluster-role-binding.yaml"
@@ -145,13 +167,29 @@ var (
 	TelemeterClientServiceAccount         = "assets/telemeter-client/service-account.yaml"
 	TelemeterClientServiceMonitor         = "assets/telemeter-client/service-monitor.yaml"
 	TelemeterClientServingCertsCABundle   = "assets/telemeter-client/serving-certs-c-a-bundle.yaml"
-	TelemeterTrustedCABundle              = "assets/telemeter-client/trusted-ca-bundle.yaml"
+
+	ThanosQuerierDeployment         = "assets/thanos-querier/deployment.yaml"
+	ThanosQuerierService            = "assets/thanos-querier/service.yaml"
+	ThanosQuerierRoute              = "assets/thanos-querier/route.yaml"
+	ThanosQuerierOauthCookieSecret  = "assets/thanos-querier/oauth-cookie-secret.yaml"
+	ThanosQuerierHtpasswdSecret     = "assets/thanos-querier/oauth-htpasswd-secret.yaml"
+	ThanosQuerierRBACProxySecret    = "assets/thanos-querier/kube-rbac-proxy-secret.yaml"
+	ThanosQuerierServiceAccount     = "assets/thanos-querier/service-account.yaml"
+	ThanosQuerierClusterRole        = "assets/thanos-querier/cluster-role.yaml"
+	ThanosQuerierClusterRoleBinding = "assets/thanos-querier/cluster-role-binding.yaml"
+	ThanosQuerierGrpcTLSSecret      = "assets/thanos-querier/grpc-tls-secret.yaml"
+	ThanosQuerierTrustedCABundle    = "assets/thanos-querier/trusted-ca-bundle.yaml"
+
+	TelemeterTrustedCABundle = "assets/telemeter-client/trusted-ca-bundle.yaml"
 )
 
 var (
-	PrometheusConfigReloaderFlag    = "--prometheus-config-reloader="
-	ConfigReloaderImageFlag         = "--config-reloader-image="
-	PrometheusOperatorNamespaceFlag = "--namespaces="
+	PrometheusConfigReloaderFlag                         = "--prometheus-config-reloader="
+	ConfigReloaderImageFlag                              = "--config-reloader-image="
+	PrometheusOperatorNamespaceFlag                      = "--namespaces="
+	PrometheusOperatorDenyNamespaceFlag                  = "--deny-namespaces="
+	PrometheusOperatorPrometheusInstanceNamespacesFlag   = "--prometheus-instance-namespaces="
+	PrometheusOperatorAlertmanagerInstanceNamespacesFlag = "--alertmanager-instance-namespaces="
 
 	AuthProxyExternalURLFlag  = "-external-url="
 	AuthProxyCookieDomainFlag = "-cookie-domain="
@@ -163,14 +201,15 @@ func MustAssetReader(asset string) io.Reader {
 }
 
 type Factory struct {
-	namespace string
-	config    *Config
+	namespace, namespaceUserWorkload string
+	config                           *Config
 }
 
-func NewFactory(namespace string, c *Config) *Factory {
+func NewFactory(namespace, namespaceUserWorkload string, c *Config) *Factory {
 	return &Factory{
-		namespace: namespace,
-		config:    c,
+		namespace:             namespace,
+		namespaceUserWorkload: namespaceUserWorkload,
+		config:                c,
 	}
 }
 
@@ -332,14 +371,15 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 
 	if trustedCABundleCM != nil {
 		volumeName := "alertmanager-trusted-ca-bundle"
-		volumePath := "/etc/pki/alertmanager-ca-bundle/"
-		for in, container := range a.Spec.Containers {
-			if container.Name == "alertmanager-proxy" {
-				a.Spec.Containers[in].VolumeMounts = append(container.VolumeMounts, trustedCABundleVolumeMount(volumeName, volumePath))
-			}
-		}
-		a.Spec.VolumeMounts = append(a.Spec.VolumeMounts, trustedCABundleVolumeMount(volumeName, volumePath))
-		a.Spec.Volumes = append(a.Spec.Volumes, trustedCABundleVolume(trustedCABundleCM.Name, volumeName))
+		a.Spec.VolumeMounts = append(a.Spec.VolumeMounts, trustedCABundleVolumeMount(volumeName))
+		volume := trustedCABundleVolume(trustedCABundleCM.Name, volumeName)
+		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
+			Key:  "ca-bundle.crt",
+			Path: "tls-ca-bundle.pem",
+		})
+		a.Spec.Volumes = append(a.Spec.Volumes, volume)
+		// We have only one container in Alertmanager CR spec and this is oauth-proxy
+		a.Spec.Containers[0].VolumeMounts = append(a.Spec.Containers[0].VolumeMounts, trustedCABundleVolumeMount(volumeName))
 	}
 	a.Namespace = f.namespace
 
@@ -588,10 +628,40 @@ func (f *Factory) PrometheusK8sClusterRoleBinding() (*rbacv1.ClusterRoleBinding,
 	return crb, nil
 }
 
+func (f *Factory) ThanosQuerierClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
+	crb, err := f.NewClusterRoleBinding(MustAssetReader(ThanosQuerierClusterRoleBinding))
+	if err != nil {
+		return nil, err
+	}
+
+	crb.Subjects[0].Namespace = f.namespace
+
+	return crb, nil
+}
+
+func (f *Factory) PrometheusUserWorkloadClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
+	crb, err := f.NewClusterRoleBinding(MustAssetReader(PrometheusUserWorkloadClusterRoleBinding))
+	if err != nil {
+		return nil, err
+	}
+
+	crb.Subjects[0].Namespace = f.namespaceUserWorkload
+
+	return crb, nil
+}
+
 func (f *Factory) PrometheusK8sClusterRole() (*rbacv1.ClusterRole, error) {
 	return f.NewClusterRole(MustAssetReader(PrometheusK8sClusterRole))
 }
 
+func (f *Factory) ThanosQuerierClusterRole() (*rbacv1.ClusterRole, error) {
+	return f.NewClusterRole(MustAssetReader(ThanosQuerierClusterRole))
+}
+
+func (f *Factory) PrometheusUserWorkloadClusterRole() (*rbacv1.ClusterRole, error) {
+	return f.NewClusterRole(MustAssetReader(PrometheusUserWorkloadClusterRole))
+}
+
 func (f *Factory) PrometheusK8sRoleConfig() (*rbacv1.Role, error) {
 	r, err := f.NewRole(MustAssetReader(PrometheusK8sRoleConfig))
 	if err != nil {
@@ -603,6 +673,17 @@ func (f *Factory) PrometheusK8sRoleConfig() (*rbacv1.Role, error) {
 	return r, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadRoleConfig() (*rbacv1.Role, error) {
+	r, err := f.NewRole(MustAssetReader(PrometheusUserWorkloadRoleConfig))
+	if err != nil {
+		return nil, err
+	}
+
+	r.Namespace = f.namespaceUserWorkload
+
+	return r, nil
+}
+
 func (f *Factory) PrometheusK8sRoleBindingList() (*rbacv1.RoleBindingList, error) {
 	rbl, err := f.NewRoleBindingList(MustAssetReader(PrometheusK8sRoleBindingList))
 	if err != nil {
@@ -616,6 +697,19 @@ func (f *Factory) PrometheusK8sRoleBindingList() (*rbacv1.RoleBindingList, error
 	return rbl, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadRoleBindingList() (*rbacv1.RoleBindingList, error) {
+	rbl, err := f.NewRoleBindingList(MustAssetReader(PrometheusUserWorkloadRoleBindingList))
+	if err != nil {
+		return nil, err
+	}
+
+	for _, rb := range rbl.Items {
+		rb.Subjects[0].Namespace = f.namespaceUserWorkload
+	}
+
+	return rbl, nil
+}
+
 func (f *Factory) PrometheusK8sRoleBindingConfig() (*rbacv1.RoleBinding, error) {
 	rb, err := f.NewRoleBinding(MustAssetReader(PrometheusK8sRoleBindingConfig))
 	if err != nil {
@@ -627,6 +721,17 @@ func (f *Factory) PrometheusK8sRoleBindingConfig() (*rbacv1.RoleBinding, error)
 	return rb, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadRoleBindingConfig() (*rbacv1.RoleBinding, error) {
+	rb, err := f.NewRoleBinding(MustAssetReader(PrometheusUserWorkloadRoleBindingConfig))
+	if err != nil {
+		return nil, err
+	}
+
+	rb.Namespace = f.namespaceUserWorkload
+
+	return rb, nil
+}
+
 func (f *Factory) PrometheusK8sRoleList() (*rbacv1.RoleList, error) {
 	rl, err := f.NewRoleList(MustAssetReader(PrometheusK8sRoleList))
 	if err != nil {
@@ -640,6 +745,19 @@ func (f *Factory) PrometheusK8sRoleList() (*rbacv1.RoleList, error) {
 	return rl, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadRoleList() (*rbacv1.RoleList, error) {
+	rl, err := f.NewRoleList(MustAssetReader(PrometheusUserWorkloadRoleList))
+	if err != nil {
+		return nil, err
+	}
+
+	for _, r := range rl.Items {
+		r.Namespace = f.namespaceUserWorkload
+	}
+
+	return rl, nil
+}
+
 func (f *Factory) PrometheusK8sRules() (*monv1.PrometheusRule, error) {
 	r, err := f.NewPrometheusRule(MustAssetReader(PrometheusK8sRules))
 	if err != nil {
@@ -672,6 +790,28 @@ func (f *Factory) PrometheusK8sServiceAccount() (*v1.ServiceAccount, error) {
 	return s, nil
 }
 
+func (f *Factory) ThanosQuerierServiceAccount() (*v1.ServiceAccount, error) {
+	s, err := f.NewServiceAccount(MustAssetReader(ThanosQuerierServiceAccount))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
+func (f *Factory) PrometheusUserWorkloadServiceAccount() (*v1.ServiceAccount, error) {
+	s, err := f.NewServiceAccount(MustAssetReader(PrometheusUserWorkloadServiceAccount))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
 func (f *Factory) PrometheusK8sProxySecret() (*v1.Secret, error) {
 	s, err := f.NewSecret(MustAssetReader(PrometheusK8sProxySecret))
 	if err != nil {
@@ -688,20 +828,82 @@ func (f *Factory) PrometheusK8sProxySecret() (*v1.Secret, error) {
 	return s, nil
 }
 
+func (f *Factory) PrometheusK8sGrpcTLSSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(PrometheusK8sGrpcTLSSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
+func (f *Factory) PrometheusUserWorkloadGrpcTLSSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(PrometheusUserWorkloadGrpcTLSSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
+func (f *Factory) ThanosQuerierGrpcTLSSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosQuerierGrpcTLSSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
+func (f *Factory) ThanosQuerierOauthCookieSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosQuerierOauthCookieSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	p, err := GeneratePassword(43)
+	if err != nil {
+		return nil, err
+	}
+	s.Data["session_secret"] = []byte(p)
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
 func (f *Factory) PrometheusK8sHtpasswdSecret(password string) (*v1.Secret, error) {
 	s, err := f.NewSecret(MustAssetReader(PrometheusK8sHtpasswd))
 	if err != nil {
 		return nil, err
 	}
 
+	f.generateHtpasswdSecret(s, password)
+	return s, nil
+}
+
+func (f *Factory) ThanosQuerierHtpasswdSecret(password string) (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosQuerierHtpasswdSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	f.generateHtpasswdSecret(s, password)
+	return s, nil
+}
+
+func (f *Factory) generateHtpasswdSecret(s *v1.Secret, password string) {
 	// #nosec
 	// TODO: Replace this with a safer algorithm
 	h := sha1.New()
 	h.Write([]byte(password))
 	s.Data["auth"] = []byte("internal:{SHA}" + base64.StdEncoding.EncodeToString(h.Sum(nil)))
 	s.Namespace = f.namespace
-
-	return s, nil
 }
 
 func (f *Factory) PrometheusRBACProxySecret() (*v1.Secret, error) {
@@ -715,6 +917,17 @@ func (f *Factory) PrometheusRBACProxySecret() (*v1.Secret, error) {
 	return s, nil
 }
 
+func (f *Factory) ThanosQuerierRBACProxySecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosQuerierRBACProxySecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
 func (f *Factory) PrometheusK8sServingCertsCABundle() (*v1.ConfigMap, error) {
 	c, err := f.NewConfigMap(MustAssetReader(PrometheusK8sServingCertsCABundle))
 	if err != nil {
@@ -726,6 +939,17 @@ func (f *Factory) PrometheusK8sServingCertsCABundle() (*v1.ConfigMap, error) {
 	return c, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadServingCertsCABundle() (*v1.ConfigMap, error) {
+	c, err := f.NewConfigMap(MustAssetReader(PrometheusUserWorkloadServingCertsCABundle))
+	if err != nil {
+		return nil, err
+	}
+
+	c.Namespace = f.namespaceUserWorkload
+
+	return c, nil
+}
+
 func (f *Factory) PrometheusK8sKubeletServingCABundle(data map[string]string) (*v1.ConfigMap, error) {
 	c, err := f.NewConfigMap(MustAssetReader(PrometheusK8sKubeletServingCABundle))
 	if err != nil {
@@ -799,7 +1023,22 @@ func (f *Factory) PrometheusK8sRoute() (*routev1.Route, error) {
 	return r, nil
 }
 
-func (f *Factory) SharingConfig(promHost, amHost, grafanaHost *url.URL) *v1.ConfigMap {
+func (f *Factory) ThanosQuerierRoute() (*routev1.Route, error) {
+	r, err := f.NewRoute(MustAssetReader(ThanosQuerierRoute))
+	if err != nil {
+		return nil, err
+	}
+
+	// apply hostport configuration to thanos
+	if f.config.PrometheusK8sConfig.Hostport != "" {
+		r.Spec.Host = f.config.PrometheusK8sConfig.Hostport
+	}
+	r.Namespace = f.namespace
+
+	return r, nil
+}
+
+func (f *Factory) SharingConfig(promHost, amHost, grafanaHost, thanosHost *url.URL) *v1.ConfigMap {
 	return &v1.ConfigMap{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "sharing-config",
@@ -809,6 +1048,7 @@ func (f *Factory) SharingConfig(promHost, amHost, grafanaHost *url.URL) *v1.Conf
 			"grafanaURL":      grafanaHost.String(),
 			"prometheusURL":   promHost.String(),
 			"alertmanagerURL": amHost.String(),
+			"thanosURL":       thanosHost.String(),
 		},
 	}
 }
@@ -822,7 +1062,17 @@ func (f *Factory) PrometheusK8sTrustedCABundle() (*v1.ConfigMap, error) {
 	return cm, nil
 }
 
-func (f *Factory) PrometheusK8s(host string, trustedCABundleCM *v1.ConfigMap) (*monv1.Prometheus, error) {
+const (
+	// These constants refer to indices of prometheus-k8s containers.
+	// They need to be in sync with jsonnet/prometheus.jsonnet
+	K8S_CONTAINER_OAUTH_PROXY      = 0
+	K8S_CONTAINER_KUBE_RBAC_PROXY  = 1
+	K8S_CONTAINER_PROM_LABEL_PROXY = 2
+	K8S_CONTAINER_THANOS_SIDECAR   = 3
+	K8S_CONTAINER_PROMETHEUS       = 4
+)
+
+func (f *Factory) PrometheusK8s(host string, grpcTLS *v1.Secret, trustedCABundleCM *v1.ConfigMap) (*monv1.Prometheus, error) {
 	p, err := f.NewPrometheus(MustAssetReader(PrometheusK8s))
 	if err != nil {
 		return nil, err
@@ -872,17 +1122,22 @@ func (f *Factory) PrometheusK8s(host string, trustedCABundleCM *v1.ConfigMap) (*
 		p.Spec.Secrets = secrets
 	}
 
-	p.Spec.Containers[0].Image = f.config.Images.OauthProxy
-	p.Spec.Containers[1].Image = f.config.Images.KubeRbacProxy
-	p.Spec.Containers[2].Image = f.config.Images.PromLabelProxy
+	if f.config.Images.Thanos != "" {
+		p.Spec.Thanos.Image = &f.config.Images.Thanos
+	}
+
+	p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].Image = f.config.Images.OauthProxy
+	p.Spec.Containers[K8S_CONTAINER_KUBE_RBAC_PROXY].Image = f.config.Images.KubeRbacProxy
+	p.Spec.Containers[K8S_CONTAINER_PROM_LABEL_PROXY].Image = f.config.Images.PromLabelProxy
+
 	p.Spec.Alerting.Alertmanagers[0].Namespace = f.namespace
 	p.Spec.Alerting.Alertmanagers[0].TLSConfig.ServerName = fmt.Sprintf("alertmanager-main.%s.svc", f.namespace)
 	p.Namespace = f.namespace
 
 	setEnv := func(name, value string) {
-		for i := range p.Spec.Containers[0].Env {
-			if p.Spec.Containers[0].Env[i].Name == name {
-				p.Spec.Containers[0].Env[i].Value = value
+		for i := range p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].Env {
+			if p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].Env[i].Name == name {
+				p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].Env[i].Value = value
 				break
 			}
 		}
@@ -897,18 +1152,98 @@ func (f *Factory) PrometheusK8s(host string, trustedCABundleCM *v1.ConfigMap) (*
 		setEnv("NO_PROXY", f.config.HTTPConfig.NoProxy)
 	}
 
+	p.Spec.Volumes = append(p.Spec.Volumes, v1.Volume{
+		Name: "secret-grpc-tls",
+		VolumeSource: v1.VolumeSource{
+			Secret: &v1.SecretVolumeSource{
+				SecretName: grpcTLS.GetName(),
+			},
+		},
+	})
+
 	if trustedCABundleCM != nil {
 		volumeName := "prometheus-trusted-ca-bundle"
-		volumePath := "/etc/pki/prometheus-ca-bundle/"
-		p.Spec.Volumes = append(p.Spec.Volumes, trustedCABundleVolume(trustedCABundleCM.Name, volumeName))
+		volume := trustedCABundleVolume(trustedCABundleCM.Name, volumeName)
+		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
+			Key:  "ca-bundle.crt",
+			Path: "tls-ca-bundle.pem",
+		})
+		p.Spec.Volumes = append(p.Spec.Volumes, volume)
 
-		for in, container := range p.Spec.Containers {
-			if container.Name == "prometheus-proxy" || container.Name == "prometheus-k8s" {
-				p.Spec.Containers[in].VolumeMounts = append(container.VolumeMounts, trustedCABundleVolumeMount(volumeName, volumePath))
-			}
+		// we only need the trusted CA bundle in:
+		// 1. Prometheus, because users might want to configure external remote write.
+		// 2. In OAuth proxy, as that communicates externally when executing the OAuth handshake.
+
+		p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].VolumeMounts = append(
+			p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].VolumeMounts,
+			trustedCABundleVolumeMount(volumeName),
+		)
+
+		p.Spec.Containers[K8S_CONTAINER_PROMETHEUS].VolumeMounts = append(
+			p.Spec.Containers[K8S_CONTAINER_PROMETHEUS].VolumeMounts,
+			trustedCABundleVolumeMount(volumeName),
+		)
+	}
+
+	return p, nil
+}
+
+func (f *Factory) PrometheusUserWorkload(grpcTLS *v1.Secret) (*monv1.Prometheus, error) {
+	p, err := f.NewPrometheus(MustAssetReader(PrometheusUserWorkload))
+	if err != nil {
+		return nil, err
+	}
+
+	if f.config.PrometheusUserWorkloadConfig.Retention != "" {
+		p.Spec.Retention = f.config.PrometheusUserWorkloadConfig.Retention
+	}
+
+	p.Spec.Image = &f.config.Images.Prometheus
+
+	if f.config.PrometheusUserWorkloadConfig.Resources != nil {
+		p.Spec.Resources = *f.config.PrometheusUserWorkloadConfig.Resources
+	}
+
+	if f.config.PrometheusUserWorkloadConfig.NodeSelector != nil {
+		p.Spec.NodeSelector = f.config.PrometheusUserWorkloadConfig.NodeSelector
+	}
+
+	if len(f.config.PrometheusUserWorkloadConfig.Tolerations) > 0 {
+		p.Spec.Tolerations = f.config.PrometheusUserWorkloadConfig.Tolerations
+	}
+
+	if f.config.PrometheusUserWorkloadConfig.ExternalLabels != nil {
+		p.Spec.ExternalLabels = f.config.PrometheusUserWorkloadConfig.ExternalLabels
+	}
+
+	if f.config.PrometheusUserWorkloadConfig.VolumeClaimTemplate != nil {
+		p.Spec.Storage = &monv1.StorageSpec{
+			VolumeClaimTemplate: *f.config.PrometheusUserWorkloadConfig.VolumeClaimTemplate,
 		}
 	}
 
+	if len(f.config.PrometheusUserWorkloadConfig.RemoteWrite) > 0 {
+		p.Spec.RemoteWrite = f.config.PrometheusUserWorkloadConfig.RemoteWrite
+	}
+
+	if f.config.Images.Thanos != "" {
+		p.Spec.Thanos.Image = &f.config.Images.Thanos
+	}
+
+	p.Spec.Containers[0].Image = f.config.Images.KubeRbacProxy
+	p.Spec.Alerting.Alertmanagers[0].Namespace = f.namespace
+	p.Spec.Alerting.Alertmanagers[0].TLSConfig.ServerName = fmt.Sprintf("alertmanager-main.%s.svc", f.namespace)
+	p.Namespace = f.namespaceUserWorkload
+
+	p.Spec.Volumes = append(p.Spec.Volumes, v1.Volume{
+		Name: "secret-grpc-tls",
+		VolumeSource: v1.VolumeSource{
+			Secret: &v1.SecretVolumeSource{
+				SecretName: grpcTLS.GetName(),
+			},
+		},
+	})
+
 	return p, nil
 }
 
@@ -935,6 +1270,18 @@ func (f *Factory) PrometheusK8sPrometheusServiceMonitor() (*monv1.ServiceMonitor
 	return sm, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadPrometheusServiceMonitor() (*monv1.ServiceMonitor, error) {
+	sm, err := f.NewServiceMonitor(MustAssetReader(PrometheusUserWorkloadPrometheusServiceMonitor))
+	if err != nil {
+		return nil, err
+	}
+
+	sm.Spec.Endpoints[0].TLSConfig.ServerName = fmt.Sprintf("prometheus-user-workload.%s.svc", f.namespaceUserWorkload)
+	sm.Namespace = f.namespaceUserWorkload
+
+	return sm, nil
+}
+
 func (f *Factory) PrometheusAdapterClusterRole() (*rbacv1.ClusterRole, error) {
 	return f.NewClusterRole(MustAssetReader(PrometheusAdapterClusterRole))
 }
@@ -1162,6 +1509,17 @@ func (f *Factory) PrometheusOperatorServiceMonitor() (*monv1.ServiceMonitor, err
 	return sm, nil
 }
 
+func (f *Factory) PrometheusOperatorUserWorkloadServiceMonitor() (*monv1.ServiceMonitor, error) {
+	sm, err := f.NewServiceMonitor(MustAssetReader(PrometheusOperatorUserWorkloadServiceMonitor))
+	if err != nil {
+		return nil, err
+	}
+
+	sm.Namespace = f.namespaceUserWorkload
+
+	return sm, nil
+}
+
 func (f *Factory) PrometheusOperatorClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
 	crb, err := f.NewClusterRoleBinding(MustAssetReader(PrometheusOperatorClusterRoleBinding))
 	if err != nil {
@@ -1173,10 +1531,25 @@ func (f *Factory) PrometheusOperatorClusterRoleBinding() (*rbacv1.ClusterRoleBin
 	return crb, nil
 }
 
+func (f *Factory) PrometheusOperatorUserWorkloadClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
+	crb, err := f.NewClusterRoleBinding(MustAssetReader(PrometheusOperatorUserWorkloadClusterRoleBinding))
+	if err != nil {
+		return nil, err
+	}
+
+	crb.Subjects[0].Namespace = f.namespaceUserWorkload
+
+	return crb, nil
+}
+
 func (f *Factory) PrometheusOperatorClusterRole() (*rbacv1.ClusterRole, error) {
 	return f.NewClusterRole(MustAssetReader(PrometheusOperatorClusterRole))
 }
 
+func (f *Factory) PrometheusOperatorUserWorkloadClusterRole() (*rbacv1.ClusterRole, error) {
+	return f.NewClusterRole(MustAssetReader(PrometheusOperatorUserWorkloadClusterRole))
+}
+
 func (f *Factory) PrometheusOperatorServiceAccount() (*v1.ServiceAccount, error) {
 	s, err := f.NewServiceAccount(MustAssetReader(PrometheusOperatorServiceAccount))
 	if err != nil {
@@ -1188,6 +1561,17 @@ func (f *Factory) PrometheusOperatorServiceAccount() (*v1.ServiceAccount, error)
 	return s, nil
 }
 
+func (f *Factory) PrometheusOperatorUserWorkloadServiceAccount() (*v1.ServiceAccount, error) {
+	s, err := f.NewServiceAccount(MustAssetReader(PrometheusOperatorUserWorkloadServiceAccount))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
 func (f *Factory) PrometheusOperatorDeployment(namespaces []string) (*appsv1.Deployment, error) {
 	d, err := f.NewDeployment(MustAssetReader(PrometheusOperatorDeployment))
 	if err != nil {
@@ -1217,6 +1601,14 @@ func (f *Factory) PrometheusOperatorDeployment(namespaces []string) (*appsv1.Dep
 		if strings.HasPrefix(args[i], ConfigReloaderImageFlag) {
 			args[i] = ConfigReloaderImageFlag + f.config.Images.ConfigmapReloader
 		}
+
+		if strings.HasPrefix(args[i], PrometheusOperatorAlertmanagerInstanceNamespacesFlag) {
+			args[i] = PrometheusOperatorAlertmanagerInstanceNamespacesFlag + f.namespace
+		}
+
+		if strings.HasPrefix(args[i], PrometheusOperatorPrometheusInstanceNamespacesFlag) {
+			args[i] = PrometheusOperatorPrometheusInstanceNamespacesFlag + f.namespace
+		}
 	}
 	d.Spec.Template.Spec.Containers[0].Args = args
 	d.Namespace = f.namespace
@@ -1224,6 +1616,50 @@ func (f *Factory) PrometheusOperatorDeployment(namespaces []string) (*appsv1.Dep
 	return d, nil
 }
 
+func (f *Factory) PrometheusOperatorUserWorkloadDeployment(denyNamespaces []string) (*appsv1.Deployment, error) {
+	d, err := f.NewDeployment(MustAssetReader(PrometheusOperatorUserWorkloadDeployment))
+	if err != nil {
+		return nil, err
+	}
+
+	if len(f.config.PrometheusOperatorUserWorkloadConfig.NodeSelector) > 0 {
+		d.Spec.Template.Spec.NodeSelector = f.config.PrometheusOperatorUserWorkloadConfig.NodeSelector
+	}
+
+	if len(f.config.PrometheusOperatorUserWorkloadConfig.Tolerations) > 0 {
+		d.Spec.Template.Spec.Tolerations = f.config.PrometheusOperatorUserWorkloadConfig.Tolerations
+	}
+
+	d.Spec.Template.Spec.Containers[0].Image = f.config.Images.PrometheusOperator
+
+	args := d.Spec.Template.Spec.Containers[0].Args
+	for i := range args {
+		if strings.HasPrefix(args[i], PrometheusOperatorDenyNamespaceFlag) {
+			args[i] = PrometheusOperatorDenyNamespaceFlag + strings.Join(denyNamespaces, ",")
+		}
+
+		if strings.HasPrefix(args[i], PrometheusConfigReloaderFlag) {
+			args[i] = PrometheusConfigReloaderFlag + f.config.Images.PrometheusConfigReloader
+		}
+
+		if strings.HasPrefix(args[i], ConfigReloaderImageFlag) {
+			args[i] = ConfigReloaderImageFlag + f.config.Images.ConfigmapReloader
+		}
+
+		if strings.HasPrefix(args[i], PrometheusOperatorAlertmanagerInstanceNamespacesFlag) {
+			args[i] = PrometheusOperatorAlertmanagerInstanceNamespacesFlag + f.namespaceUserWorkload
+		}
+
+		if strings.HasPrefix(args[i], PrometheusOperatorPrometheusInstanceNamespacesFlag) {
+			args[i] = PrometheusOperatorPrometheusInstanceNamespacesFlag + f.namespaceUserWorkload
+		}
+	}
+	d.Spec.Template.Spec.Containers[0].Args = args
+	d.Namespace = f.namespaceUserWorkload
+
+	return d, nil
+}
+
 func (f *Factory) PrometheusOperatorService() (*v1.Service, error) {
 	s, err := f.NewService(MustAssetReader(PrometheusOperatorService))
 	if err != nil {
@@ -1235,6 +1671,17 @@ func (f *Factory) PrometheusOperatorService() (*v1.Service, error) {
 	return s, nil
 }
 
+func (f *Factory) PrometheusOperatorUserWorkloadService() (*v1.Service, error) {
+	s, err := f.NewService(MustAssetReader(PrometheusOperatorUserWorkloadService))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
 func (f *Factory) PrometheusK8sService() (*v1.Service, error) {
 	s, err := f.NewService(MustAssetReader(PrometheusK8sService))
 	if err != nil {
@@ -1246,6 +1693,17 @@ func (f *Factory) PrometheusK8sService() (*v1.Service, error) {
 	return s, nil
 }
 
+func (f *Factory) PrometheusUserWorkloadService() (*v1.Service, error) {
+	s, err := f.NewService(MustAssetReader(PrometheusUserWorkloadService))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
 func (f *Factory) GrafanaClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
 	crb, err := f.NewClusterRoleBinding(MustAssetReader(GrafanaClusterRoleBinding))
 	if err != nil {
@@ -1423,11 +1881,7 @@ func (f *Factory) GrafanaDeployment(proxyCABundleCM *v1.ConfigMap) (*appsv1.Depl
 
 	if proxyCABundleCM != nil {
 		volumeName := "grafana-trusted-ca-bundle"
-		for in, container := range d.Spec.Template.Spec.Containers {
-			if container.Name == "grafana-proxy" {
-				d.Spec.Template.Spec.Containers[in].VolumeMounts = append(d.Spec.Template.Spec.Containers[in].VolumeMounts, trustedCABundleVolumeMount(volumeName, "/etc/pki/ca-trust/extracted/pem/"))
-			}
-		}
+		d.Spec.Template.Spec.Containers[1].VolumeMounts = append(d.Spec.Template.Spec.Containers[1].VolumeMounts, trustedCABundleVolumeMount(volumeName))
 		volume := trustedCABundleVolume(proxyCABundleCM.Name, volumeName)
 		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
 			Key:  "ca-bundle.crt",
@@ -1815,6 +2269,100 @@ func (f *Factory) NewClusterRole(manifest io.Reader) (*rbacv1.ClusterRole, error
 	return NewClusterRole(manifest)
 }
 
+const (
+	// These constants refer to indices of prometheus-k8s containers.
+	// They need to be in sync with jsonnet/prometheus.jsonnet
+	THANOS_QUERIER_CONTAINER_THANOS           = 0
+	THANOS_QUERIER_CONTAINER_OAUTH_PROXY      = 1
+	THANOS_QUERIER_CONTAINER_KUBE_RBAC_PROXY  = 2
+	THANOS_QUERIER_CONTAINER_PROM_LABEL_PROXY = 3
+)
+
+func (f *Factory) ThanosQuerierDeployment(grpcTLS *v1.Secret, enableUserWorkloadMonitoring bool, trustedCA *v1.ConfigMap) (*appsv1.Deployment, error) {
+	d, err := f.NewDeployment(MustAssetReader(ThanosQuerierDeployment))
+	if err != nil {
+		return nil, err
+	}
+
+	setEnv := func(name, value string) {
+		for i := range d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_OAUTH_PROXY].Env {
+			if d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_OAUTH_PROXY].Env[i].Name == name {
+				d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_OAUTH_PROXY].Env[i].Value = value
+				break
+			}
+		}
+	}
+
+	if f.config.HTTPConfig.HTTPProxy != "" {
+		setEnv("HTTP_PROXY", f.config.HTTPConfig.HTTPProxy)
+	}
+	if f.config.HTTPConfig.HTTPSProxy != "" {
+		setEnv("HTTPS_PROXY", f.config.HTTPConfig.HTTPSProxy)
+	}
+	if f.config.HTTPConfig.NoProxy != "" {
+		setEnv("NO_PROXY", f.config.HTTPConfig.NoProxy)
+	}
+
+	d.Namespace = f.namespace
+	d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_THANOS].Image = f.config.Images.Thanos
+	d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_OAUTH_PROXY].Image = f.config.Images.OauthProxy
+	d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_KUBE_RBAC_PROXY].Image = f.config.Images.KubeRbacProxy
+	d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_PROM_LABEL_PROXY].Image = f.config.Images.PromLabelProxy
+
+	if enableUserWorkloadMonitoring {
+		d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_THANOS].Args = append(
+			d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_THANOS].Args,
+			"--store=dnssrv+_grpc._tcp.prometheus-operated.openshift-user-workload-monitoring.svc.cluster.local",
+		)
+	}
+
+	d.Spec.Template.Spec.Volumes = append(d.Spec.Template.Spec.Volumes, v1.Volume{
+		Name: "secret-grpc-tls",
+		VolumeSource: v1.VolumeSource{
+			Secret: &v1.SecretVolumeSource{
+				SecretName: grpcTLS.GetName(),
+			},
+		},
+	})
+
+	if trustedCA != nil {
+		volumeName := "thanos-querier-trusted-ca-bundle"
+		d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_OAUTH_PROXY].VolumeMounts = append(
+			d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_OAUTH_PROXY].VolumeMounts,
+			trustedCABundleVolumeMount(volumeName),
+		)
+
+		volume := trustedCABundleVolume(trustedCA.Name, volumeName)
+		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
+			Key:  "ca-bundle.crt",
+			Path: "tls-ca-bundle.pem",
+		})
+		d.Spec.Template.Spec.Volumes = append(d.Spec.Template.Spec.Volumes, volume)
+	}
+
+	return d, nil
+}
+
+func (f *Factory) ThanosQuerierTrustedCABundle() (*v1.ConfigMap, error) {
+	cm, err := f.NewConfigMap(MustAssetReader(ThanosQuerierTrustedCABundle))
+	if err != nil {
+		return nil, err
+	}
+
+	return cm, nil
+}
+
+func (f *Factory) ThanosQuerierService() (*v1.Service, error) {
+	s, err := f.NewService(MustAssetReader(ThanosQuerierService))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
 func (f *Factory) TelemeterTrustedCABundle() (*v1.ConfigMap, error) {
 	cm, err := f.NewConfigMap(MustAssetReader(TelemeterTrustedCABundle))
 	if err != nil {
@@ -1927,7 +2475,7 @@ func (f *Factory) TelemeterClientDeployment(proxyCABundleCM *v1.ConfigMap) (*app
 	d.Namespace = f.namespace
 	if proxyCABundleCM != nil {
 		volumeName := "telemeter-trusted-ca-bundle"
-		d.Spec.Template.Spec.Containers[0].VolumeMounts = append(d.Spec.Template.Spec.Containers[0].VolumeMounts, trustedCABundleVolumeMount(volumeName, "/etc/pki/ca-trust/extracted/pem/"))
+		d.Spec.Template.Spec.Containers[0].VolumeMounts = append(d.Spec.Template.Spec.Containers[0].VolumeMounts, trustedCABundleVolumeMount(volumeName))
 		volume := trustedCABundleVolume(proxyCABundleCM.Name, volumeName)
 		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
 			Key:  "ca-bundle.crt",
@@ -2239,11 +2787,50 @@ func (f *Factory) HashTrustedCA(caBundleCM *v1.ConfigMap, prefix string) *v1.Con
 	}
 }
 
-func trustedCABundleVolumeMount(name, path string) v1.VolumeMount {
+// HashSecret synthesizes a secret by setting the given data
+// and naming it by hashing the values of the given data.
+//
+// For simplicity, data is expected to be given in a key-value format,
+// i.e. HashSecret(someSecret, value1, key1, value2, key2, ...).
+//
+// It adds "monitoring.openshift.io/name" and "monitoring.openshift.io/hash" labels.
+// Any other labels from the given secret are discarded.
+//
+// It still returns a secret if the given secret does not contain any data.
+func (f *Factory) HashSecret(secret *v1.Secret, data ...string) (*v1.Secret, error) {
+	h := fnv.New64()
+	m := make(map[string][]byte)
+
+	var err error
+	for i := 0; i < len(data)/2; i++ {
+		k := data[i*2]
+		v := []byte(data[i*2+1])
+		_, err = h.Write(v)
+		m[k] = v
+	}
+	if err != nil {
+		return nil, errors.Wrap(err, "error hashing tls data")
+	}
+	hash := strconv.FormatUint(h.Sum64(), 32)
+
+	return &v1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: secret.GetNamespace(),
+			Name:      fmt.Sprintf("%s-%s", secret.GetName(), hash),
+			Labels: map[string]string{
+				"monitoring.openshift.io/name": secret.GetName(),
+				"monitoring.openshift.io/hash": hash,
+			},
+		},
+		Data: m,
+	}, nil
+}
+
+func trustedCABundleVolumeMount(name string) v1.VolumeMount {
 	return v1.VolumeMount{
 		Name:      name,
 		ReadOnly:  true,
-		MountPath: path,
+		MountPath: "/etc/pki/ca-trust/extracted/pem/",
 	}
 }
 
