@@ -23,13 +23,162 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
 
+func TestHashSecret(t *testing.T) {
+	for _, tt := range []struct {
+		name            string
+		data            []string
+		given, expected *v1.Secret
+		errExpected     bool
+	}{
+		{
+			name:  "no data",
+			given: &v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}},
+			expected: &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "foo-cnskssi2248p5",
+					Labels: map[string]string{
+						"monitoring.openshift.io/hash": "cnskssi2248p5",
+						"monitoring.openshift.io/name": "foo",
+					},
+				},
+				Data: make(map[string][]byte),
+			},
+		},
+		{
+			name:  "one entry",
+			given: &v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}},
+			data:  []string{"key1", "value1"},
+			expected: &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "foo-3dquk0q6eln15",
+					Labels: map[string]string{
+						"monitoring.openshift.io/hash": "3dquk0q6eln15",
+						"monitoring.openshift.io/name": "foo",
+					},
+				},
+				Data: map[string][]byte{
+					"key1": []byte("value1"),
+				},
+			},
+		},
+		{
+			name:  "one valid one invalid entry",
+			given: &v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}},
+			data:  []string{"key1", "value1", "key2"},
+			expected: &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "foo-3dquk0q6eln15",
+					Labels: map[string]string{
+						"monitoring.openshift.io/hash": "3dquk0q6eln15",
+						"monitoring.openshift.io/name": "foo",
+					},
+				},
+				Data: map[string][]byte{
+					"key1": []byte("value1"),
+				},
+			},
+		},
+		{
+			name:  "two entries",
+			given: &v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}},
+			data:  []string{"key1", "value1", "key2", "value2"},
+			expected: &v1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "foo-bfcd7k3kr4396",
+					Labels: map[string]string{
+						"monitoring.openshift.io/hash": "bfcd7k3kr4396",
+						"monitoring.openshift.io/name": "foo",
+					},
+				},
+				Data: map[string][]byte{
+					"key1": []byte("value1"),
+					"key2": []byte("value2"),
+				},
+			},
+		},
+	} {
+		t.Run(tt.name, func(t *testing.T) {
+			f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", NewDefaultConfig())
+			s, err := f.HashSecret(tt.given, tt.data...)
+			if got := err != nil; got != tt.errExpected {
+				t.Errorf("expected error %t, got %t, err %v", tt.errExpected, got, err)
+				return
+			}
+
+			if !reflect.DeepEqual(s, tt.expected) {
+				t.Errorf("expected secret to be equal, but it isn't. got %v, expected %v", s, tt.expected)
+			}
+		})
+	}
+}
+
 func TestUnconfiguredManifests(t *testing.T) {
-	f := NewFactory("openshift-monitoring", NewDefaultConfig())
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", NewDefaultConfig())
 	_, err := f.AlertmanagerConfig()
 	if err != nil {
 		t.Fatal(err)
 	}
 
+	_, err = f.PrometheusK8sGrpcTLSSecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadGrpcTLSSecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierGrpcTLSSecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierClusterRoleBinding()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierDeployment(&v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}}, true, nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierServiceAccount()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierOauthCookieSecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierHtpasswdSecret("foo")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierRBACProxySecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierRoute()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosQuerierService()
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	_, err = f.AlertmanagerService()
 	if err != nil {
 		t.Fatal(err)
@@ -170,7 +319,7 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	_, err = f.PrometheusK8s("prometheus-k8s.openshift-monitoring.svc", nil)
+	_, err = f.PrometheusK8s("prometheus-k8s.openshift-monitoring.svc", &v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}}, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -235,6 +384,86 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
+	_, err = f.PrometheusOperatorUserWorkloadServiceMonitor()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusOperatorUserWorkloadClusterRoleBinding()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusOperatorUserWorkloadClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusOperatorUserWorkloadServiceAccount()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusOperatorUserWorkloadService()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadServingCertsCABundle()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadServiceAccount()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadClusterRoleBinding()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadRoleConfig()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadRoleList()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadRoleBindingList()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadRoleBindingConfig()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadService()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkload(&v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.PrometheusUserWorkloadPrometheusServiceMonitor()
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	tlsSecret := &v1.Secret{
 		Data: map[string][]byte{
 			"tls.crt": []byte("foo"),
@@ -414,7 +643,7 @@ func TestPrometheusOperatorConfiguration(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 	d, err := f.PrometheusOperatorDeployment([]string{"default", "openshift-monitoring"})
 	if err != nil {
 		t.Fatal(err)
@@ -499,8 +728,12 @@ ingress:
 		"prom-label-proxy": "docker.io/openshift/origin-prom-label-proxy:latest",
 	})
 
-	f := NewFactory("openshift-monitoring", c)
-	p, err := f.PrometheusK8s("prometheus-k8s.openshift-monitoring.svc", &v1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Name: "foo"}})
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
+	p, err := f.PrometheusK8s(
+		"prometheus-k8s.openshift-monitoring.svc",
+		&v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}},
+		&v1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Name: "foo"}},
+	)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -513,15 +746,15 @@ ingress:
 		t.Fatal("Prometheus image is not configured correctly")
 	}
 
-	if p.Spec.Containers[0].Image != "docker.io/openshift/origin-oauth-proxy:latest" {
+	if p.Spec.Containers[K8S_CONTAINER_OAUTH_PROXY].Image != "docker.io/openshift/origin-oauth-proxy:latest" {
 		t.Fatal("oauth-proxy image is not configured correctly")
 	}
 
-	if p.Spec.Containers[1].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
+	if p.Spec.Containers[K8S_CONTAINER_KUBE_RBAC_PROXY].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
 		t.Fatal("kube-rbac-proxy image is not configured correctly")
 	}
 
-	if p.Spec.Containers[2].Image != "docker.io/openshift/origin-prom-label-proxy:latest" {
+	if p.Spec.Containers[K8S_CONTAINER_PROM_LABEL_PROXY].Image != "docker.io/openshift/origin-prom-label-proxy:latest" {
 		t.Fatal("prom-label-proxy image is not configured correctly")
 	}
 
@@ -584,7 +817,7 @@ k8sPrometheusAdapter:
 		"k8s-prometheus-adapter": "docker.io/openshift/origin-k8s-prometheus-adapter:latest",
 	})
 
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 	d, err := f.PrometheusAdapterDeployment("foo", map[string]string{
 		"requestheader-allowed-names":        "",
 		"requestheader-extra-headers-prefix": "",
@@ -634,7 +867,7 @@ ingress:
 		"alertmanager": "docker.io/openshift/origin-prometheus-alertmanager:latest",
 	})
 
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 	a, err := f.AlertmanagerMain("alertmanager-main.openshift-monitoring.svc", nil)
 	if err != nil {
 		t.Fatal(err)
@@ -693,7 +926,7 @@ func TestNodeExporter(t *testing.T) {
 		"kube-rbac-proxy": "docker.io/openshift/origin-kube-rbac-proxy:latest",
 	})
 
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	ds, err := f.NodeExporterDaemonSet()
 	if err != nil {
@@ -717,7 +950,7 @@ func TestKubeStateMetrics(t *testing.T) {
 		"kube-rbac-proxy":    "docker.io/openshift/origin-kube-rbac-proxy:latest",
 	})
 
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	d, err := f.KubeStateMetricsDeployment()
 	if err != nil {
@@ -745,7 +978,7 @@ func TestOpenShiftStateMetrics(t *testing.T) {
 		"kube-rbac-proxy":         "docker.io/openshift/origin-kube-rbac-proxy:latest",
 	})
 
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	d, err := f.OpenShiftStateMetricsDeployment()
 	if err != nil {
@@ -767,7 +1000,7 @@ func TestPrometheusEtcdRulesFiltered(t *testing.T) {
 	enabled := false
 	c := NewDefaultConfig()
 	c.EtcdConfig.Enabled = &enabled
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	r, err := f.PrometheusK8sRules()
 	if err != nil {
@@ -785,7 +1018,7 @@ func TestPrometheusEtcdRules(t *testing.T) {
 	enabled := true
 	c := NewDefaultConfig()
 	c.EtcdConfig.Enabled = &enabled
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	r, err := f.PrometheusK8sRules()
 	if err != nil {
@@ -807,7 +1040,7 @@ func TestEtcdGrafanaDashboardFiltered(t *testing.T) {
 	enabled := false
 	c := NewDefaultConfig()
 	c.EtcdConfig.Enabled = &enabled
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	cms, err := f.GrafanaDashboardDefinitions()
 	if err != nil {
@@ -825,7 +1058,7 @@ func TestEtcdGrafanaDashboard(t *testing.T) {
 	enabled := true
 	c := NewDefaultConfig()
 	c.EtcdConfig.Enabled = &enabled
-	f := NewFactory("openshift-monitoring", c)
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
 
 	cms, err := f.GrafanaDashboardDefinitions()
 	if err != nil {
