@@ -15,14 +15,9 @@
 package tasks
 
 import (
-	"reflect"
-
 	"github.com/openshift/cluster-monitoring-operator/pkg/client"
 	"github.com/openshift/cluster-monitoring-operator/pkg/manifests"
 	"github.com/pkg/errors"
-	apierrors "k8s.io/apimachinery/pkg/api/errors"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/klog"
 )
 
 type PrometheusOperatorTask struct {
@@ -78,34 +73,16 @@ func (t *PrometheusOperatorTask) Run() error {
 		return errors.Wrap(err, "reconciling Prometheus Operator Service failed")
 	}
 
-	namespaces, err := t.client.NamespacesToMonitor()
+	clusterNamespaces, err := t.client.NamespacesToMonitor()
 	if err != nil {
 		return errors.Wrap(err, "listing namespaces to monitor failed")
 	}
 
-	d, err := t.factory.PrometheusOperatorDeployment(namespaces)
+	d, err := t.factory.PrometheusOperatorDeployment(clusterNamespaces)
 	if err != nil {
 		return errors.Wrap(err, "initializing Prometheus Operator Deployment failed")
 	}
 
-	// TODO: this is a migration from 4.1 to 4.2, it can be removed afterwards.
-	// This is necessary as the .spec.selector of the Prometheus Operator
-	// deployment changed.
-	dep, err := t.client.KubernetesInterface().AppsV1beta2().Deployments(d.GetNamespace()).Get(d.GetName(), metav1.GetOptions{})
-	if err == nil {
-		if !reflect.DeepEqual(dep.Spec.Selector.MatchLabels, d.Spec.Selector.MatchLabels) {
-			klog.V(4).Info("Detected a prometheus-operator deployment with an outdated Pod selector. Deleting prometheus-operator deployment with outdated selector, and re-creating with new selector.")
-			propagationPolicy := metav1.DeletePropagationForeground
-			err := t.client.KubernetesInterface().AppsV1beta2().Deployments(dep.GetNamespace()).Delete(dep.GetName(), &metav1.DeleteOptions{PropagationPolicy: &propagationPolicy})
-			if err != nil {
-				return errors.Wrap(err, "deleting old prometheus-operator deployment failed")
-			}
-		}
-	}
-	if err != nil && !apierrors.IsNotFound(err) {
-		return errors.Wrap(err, "retrieving existing prometheus-operator deployment failed")
-	}
-
 	err = t.client.CreateOrUpdateDeployment(d)
 	if err != nil {
 		return errors.Wrap(err, "reconciling Prometheus Operator Deployment failed")
