@@ -38,18 +38,23 @@ import (
 	metricsclient "k8s.io/metrics/pkg/client/clientset/versioned"
 )
 
-var namespaceName = "openshift-monitoring"
+var (
+	namespaceName             = "openshift-monitoring"
+	userWorkloadNamespaceName = "openshift-user-workload-monitoring"
+)
 
 type Framework struct {
 	OperatorClient      *client.Client
 	CRDClient           crdc.CustomResourceDefinitionInterface
 	KubeClient          kubernetes.Interface
-	PrometheusK8sClient *PrometheusClient
+	ThanosQuerierClient *RouteClient
+	PrometheusK8sClient *RouteClient
+	AlertmanagerClient  *RouteClient
 	APIServicesClient   *apiservicesclient.Clientset
 	MetricsClient       *metricsclient.Clientset
 
-	MonitoringClient *monClient.MonitoringV1Client
-	Ns               string
+	MonitoringClient             *monClient.MonitoringV1Client
+	Ns, UserWorkloadMonitoringNs string
 }
 
 // New returns a new cluster monitoring operator end-to-end test framework and
@@ -98,13 +103,14 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 	}
 
 	f := &Framework{
-		OperatorClient:    operatorClient,
-		KubeClient:        kubeClient,
-		CRDClient:         crdClient,
-		APIServicesClient: apiServicesClient,
-		MetricsClient:     metricsClient,
-		MonitoringClient:  mClient,
-		Ns:                namespaceName,
+		OperatorClient:           operatorClient,
+		KubeClient:               kubeClient,
+		CRDClient:                crdClient,
+		APIServicesClient:        apiServicesClient,
+		MetricsClient:            metricsClient,
+		MonitoringClient:         mClient,
+		Ns:                       namespaceName,
+		UserWorkloadMonitoringNs: userWorkloadNamespaceName,
 	}
 
 	cleanUp, err := f.setup()
@@ -113,9 +119,28 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 	}
 
 	// Prometheus client depends on setup above.
-	f.PrometheusK8sClient, err = NewPrometheusClient(openshiftRouteClient, kubeClient)
+	f.ThanosQuerierClient, err = NewRouteClient(
+		openshiftRouteClient, kubeClient,
+		"openshift-monitoring", "thanos-querier",
+	)
 	if err != nil {
-		return nil, nil, errors.Wrap(err, "creating prometheusK8sClient failed")
+		return nil, nil, errors.Wrap(err, "creating ThanosQuerierClient failed")
+	}
+
+	f.PrometheusK8sClient, err = NewRouteClient(
+		openshiftRouteClient, kubeClient,
+		"openshift-monitoring", "prometheus-k8s",
+	)
+	if err != nil {
+		return nil, nil, errors.Wrap(err, "creating PrometheusK8sClient failed")
+	}
+
+	f.AlertmanagerClient, err = NewRouteClient(
+		openshiftRouteClient, kubeClient,
+		"openshift-monitoring", "alertmanager-main",
+	)
+	if err != nil {
+		return nil, nil, errors.Wrap(err, "creating AlertmanagerClient failed")
 	}
 
 	return f, cleanUp, nil
